// Matt Dorsett
// Matthew Kraus
// CSE466
// Lab 5, Part 2 (press button to turn on remote LED)

#include "bsp.h"
#include "mrfi.h"
#include "nwk_types.h"
#include "nwk_api.h"
#include "bsp_leds.h"
#include "bsp_buttons.h"
#include "nwk.h"
#include "queue.h"

#include "app_remap_led.h"

#define PORT_A (SMPL_PORT_STATIC_MAX - 0)
#define SPIN_ABOUT_A_SECOND  NWK_DELAY(1000)

QUEUE_DATA RF_BUFFER;
QUEUE_DATA RX_BUFFER;
static uint8_t msg_buffer[MAX_APP_PAYLOAD];
static uint8_t send_buffer[MAX_APP_PAYLOAD];
static linkID_t sLinkID2 = 0;
int frame_read;

static void linkFrom(void);
static void connect();

/* Rx callback handler */
static uint8_t sRxCallback(linkID_t);

// Receives a typed character, sends it to the other IR module
#pragma vector = USCIAB0RX_VECTOR
__interrupt void USCIAB0RX_ISR (void)
{
	uint8_t byte = UCA0RXBUF;
	if (byte == ';')
		frame_read = TRUE;
	queue_push(&RX_BUFFER, byte);
}

void uart_init()
{
	// Initializing or reconfiguring the USCI module: the recommended USCI initialization/reconfiguration process is:
	// 1. set UCSWRST (BIS.B #CSWRST, &UCAxCTL1)
	// 2. Initialize all USCI registers with UCSWRST = 1 (including UCAxCTL1)
	// 3. configure ports
	// 4. clear UCSWRST via software (BIC.B #UCSWRST, &UCAxCTL1)
	// 5. enable interrupts via UCAxRXIE and/or UCAxTXIE

	WDTCTL = WDTPW | WDTHOLD;

	P3SEL = 0x30;
	UCA0CTL0 = UCMSB;   // parity disable, odd parity, LSB first, 8-bit data, one stop bit, UART mode, asynchronous mode
	UCA0CTL1 = UCSSEL_2 | UCSWRST;  // SMCLK, software reset enable == (USCI logic held in reset state)
	UCA0BR1 = 0x03;       // USCI_Ax Baud Rade Control Register 1
	UCA0BR0 = 0x41;       // USCI_Ax Baud Rade Control Register 0, pg 435
	UCA0MCTL = UCBRS_2;  // USCI_Ax modulation control register: second modulation stage select; determine the modulation pattern (pg 442)
	UCA0CTL1 &= ~UCSWRST;  // software reset disabled: USCI reset released for operation
	queue_create(&RF_BUFFER, MAX_QUEUE_ELEMENTS);
	queue_create(&RX_BUFFER, MAX_QUEUE_ELEMENTS);
	IE2 |= UCA0RXIE;
}

void main (void)
{
	BSP_Init();

	uart_init();

	/* If an on-the-fly device address is generated it must be done before the
	 * call to SMPL_Init(). If the address is set here the ROM value will not
	 * be used. If SMPL_Init() runs before this IOCTL is used the IOCTL call
	 * will not take effect. One shot only. The IOCTL call below is conformal.
	 */
#ifdef I_WANT_TO_CHANGE_DEFAULT_ROM_DEVICE_ADDRESS_PSEUDO_CODE
	{
		addr_t lAddr;

		createRandomAddress(&lAddr);
		SMPL_Ioctl(IOCTL_OBJ_ADDR, IOCTL_ACT_SET, &lAddr);
	}
#endif /* I_WANT_TO_CHANGE_DEFAULT_ROM_DEVICE_ADDRESS_PSEUDO_CODE */

	/* This call will fail because the join will fail since there is no Access Point
	 * in this scenario. But we don't care -- just use the default link token later.
	 * We supply a callback pointer to handle the message returned by the peer.
	 */
	SMPL_Init(sRxCallback);

	connect();

	/* never coming back... */
	linkFrom();

	/* turn off LEDs */
	if (BSP_LED2_IS_ON()) BSP_TOGGLE_LED2();
	if (BSP_LED1_IS_ON()) BSP_TOGGLE_LED1();

	while (1) {
		if (frame_read || queue_size(&RX_BUFFER) >= 10) {
			int len = queue_pop_all(&RX_BUFFER, send_buffer, MAX_APP_PAYLOAD);
			SMPL_Send(sLinkID2, send_buffer, len);
			if (queue_isEmpty(&RX_BUFFER))
				frame_read = FALSE;
			BSP_TOGGLE_LED1();
		}
		if (!queue_isEmpty(&RF_BUFFER)) {
			uint8_t elem;
			while (queue_pop(&RF_BUFFER, &elem)) {
				while (!(IFG2&UCA0TXIFG));
				UCA0TXBUF = elem;
			}
		}
		__bis_SR_register(GIE);
	}
}


///////////////////////////////////////////////////////////////////////////////
// Uses the "commission" facility to make a static link. The SMPL_Commission()
// function registers the address in a table, but does NOT connect. A ping is
// continually sent out until it hears back from the other device. This STILL
// does not guarantee a connection; the ping only says that the far end device
// was powered on and found. However, this method assumes that once the device
// is found a connection will quickly and automatically be established. Once
// the far end device is found, both LEDs are flashed and this method returns.
static void connect()
{
	addr_t peerAddr;
	// LinkTo Address 0xE8, 0x56, 0x34, 0x12}
	peerAddr.addr[0] = 0xE8;
	peerAddr.addr[1] = 0x56;
	peerAddr.addr[2] = 0x34;
	peerAddr.addr[3] = 0x12;

	while (SMPL_Commission(&peerAddr, PORT_A, PORT_A, &sLinkID2) != SMPL_SUCCESS)
	{
		NWK_DELAY(300);
	}

	// wait for other device to come up
	while (SMPL_Ping(sLinkID2) != SMPL_SUCCESS)
	{
		NWK_DELAY(100);
	}

	// flash lights
	BSP_TOGGLE_LED1();    // on
	BSP_TOGGLE_LED2();
	NWK_DELAY(300);
	BSP_TOGGLE_LED1();    // off
	BSP_TOGGLE_LED2();

} // end connect()

static void linkFrom()
{
	/* turn on RX. default is RX off. */
	SMPL_Ioctl( IOCTL_OBJ_RADIO, IOCTL_ACT_RADIO_RXON, 0);
}

/* handle received messages */
static uint8_t sRxCallback(linkID_t port)
{
	uint8_t len = 0;

	/* is the callback for the link ID we want to handle? */
	if (port == sLinkID2)
	{
		/* yes. go get the frame. we know this call will succeed. */
		if ((SMPL_SUCCESS == SMPL_Receive(sLinkID2, msg_buffer, &len)) && len)
		{
			int i;
			for (i = 0; i < len; i++) {
				queue_push(&RF_BUFFER, msg_buffer[i]);
			}
			BSP_TOGGLE_LED2();
			return 1;
		}
	}
	/* keep frame for later handling */
	return 0;
}

